"""
This module includes a lot of enum classes to
include a single source of truth and avoid
having names and variables hardcoded in the
project.

There are some private Enum classes (they start
with '_') to reuse the names in the other Enum
classes that are expected to be used in the 
different modules of the app.
"""
